//
//  AlamofireLoaderTask.swift
//  schedule-alamofire
//
//  Created by Илья Лошкарёв on 07.04.2018.
//  Copyright © 2018 SFEDU. All rights reserved.
//

import Foundation
import Alamofire

enum DataLoaderError: Error {
    /// Ошибка сессии (локальная)
    case sessionError(localizedDescription: String)
    /// Некорректный ответ сервера
    case responseError(response: URLResponse?)
    /// Неправильный формат данных
    case dataError(data: Data?)
}

protocol AlamofireLoaderTask {
    /// Тип получаемых данных
    associatedtype DataType
    
    /// Источник данных
    var source: URLConvertible {get}
    
    /// Инициализатор от источника
    init(from url: URLConvertible)
    
    /// Преобразование полученных данных в необходимый тип
    func decode(_ data: Data?) -> DataType?
    
    /// Выполнить запрос в указанной сессии
    ///     sessionManager - сессия, в рамках которой будет происходить загрузка
    ///     completionHandler - обработчик успешно загруженных данных
    ///     errorHandler - обработчик ошибок
    func perform(in sessionManager: Session,
                 withCompletion completionHandler: @escaping (DataType)->(),
                 withError errorHandler: ((DataLoaderError)->())?)
}
