//
//  StubDataLoader.swift
//  schedule-connect
//
//  Created by Илья Лошкарёв on 12.04.2018.
//  Copyright © 2018 SFEDU. All rights reserved.
//

import Foundation
import Alamofire

class StubDataLoader: AlamofireLoaderTask {
    typealias DataType = StubSchedule
    
    let source: URLConvertible
    
    required init(from url: URLConvertible) {
        source = url
    }
    
    func decode(_ data: Data?) -> StubSchedule? {
        guard let data = data else {return nil}
        return try? JSONDecoder().decode(StubSchedule.self, from: data)
    }
    
    func perform(in sessionManager: Session, withCompletion completionHandler: @escaping (StubSchedule) -> (), withError errorHandler: ((DataLoaderError) -> ())?) {
        
        sessionManager.request(source).validate().response {
            resp in
            switch resp.result {
            case .success(let data):
                if let decoded = self.decode(data) {
                    completionHandler(decoded)
                } else {
                    errorHandler?(.dataError(data: data))
                }
            case .failure(let error):
                if error.isResponseValidationError {
                    errorHandler?(.responseError(response: resp.response!))
                } else {
                    errorHandler?(.sessionError(localizedDescription: error.localizedDescription))
                }
                
            }
        }
    }
}
